@@ -1,5 +1,7 @@
 """Device Manager keeps track of the current state of all devices."""
 
+from __future__ import annotations
+
 from typing import Dict, Optional
 
 from .device import Device
@@ -11,11 +13,11 @@ from .structure import InfoTrait, RoomInfoTrait, Structure
 class DeviceManager:
     """DeviceManager holds current state of all devices."""
 
-    def __init__(self) -> None:
+    def __init__(self, cache_policy: CachePolicy | None = None) -> None:
         """Initialize DeviceManager."""
         self._devices: Dict[str, Device] = {}
         self._structures: Dict[str, Structure] = {}
-        self._cache_policy = CachePolicy()
+        self._cache_policy = cache_policy if cache_policy else CachePolicy()
 
     @property
     def devices(self) -> Dict[str, Device]:
