@@ -18,6 +18,7 @@ from google.protobuf.duration_pb2 import Duration
 from .auth import AbstractAuth
 from .device_manager import DeviceManager
 from .event import EventMessage
+from .event_media import CachePolicy
 from .exceptions import AuthException, ConfigurationException, SubscriberException
 from .google_nest_api import GoogleNestAPI
 
@@ -42,6 +43,16 @@ DEFAULT_MESSAGE_RETENTION_SECONDS = 15 * 60  # 15 minutes
 # Note: Users of non-prod instances will have to manually configure a topic
 TOPIC_FORMAT = "projects/sdm-prod/topics/enterprise-{project_id}"
 
+OAUTH2_AUTHORIZE_FORMAT = (
+    "https://nestservices.google.com/partnerconnections/{project_id}/auth"
+)
+OAUTH2_TOKEN = "https://www.googleapis.com/oauth2/v4/token"
+SDM_SCOPES = [
+    "https://www.googleapis.com/auth/sdm.service",
+    "https://www.googleapis.com/auth/pubsub",
+]
+API_URL = "https://smartdevicemanagement.googleapis.com/v1"
+
 
 def _validate_subscription_name(subscription_name: str) -> None:
     """Validates that a subcription name is correct.
@@ -275,6 +286,7 @@ class GoogleNestSubscriber:
         self._watchdog_restart_delay_min_seconds = watchdog_restart_delay_min_seconds
         self._watchdog_restart_delay_seconds = watchdog_restart_delay_min_seconds
         self._watchdog_task: Optional[asyncio.Task] = None
+        self._cache_policy = CachePolicy()
 
     @property
     def subscriber_id(self) -> str:
@@ -393,11 +405,6 @@ class GoogleNestSubscriber:
                 await self.start_async()
             await asyncio.sleep(self._watchdog_check_interval_seconds)
 
-    def wait(self) -> None:
-        """Block on the subscriber."""
-        assert self._subscriber_future
-        self._subscriber_future.result()
-
     def stop_async(self) -> None:
         """Tell the subscriber to start shutting down."""
         if self._device_manager_task:
@@ -407,6 +414,11 @@ class GoogleNestSubscriber:
         if self._subscriber_future:
             self._subscriber_future.cancel()
 
+    @property
+    def cache_policy(self) -> CachePolicy:
+        """Return cache policy shared by device EventMediaManager objects."""
+        return self._cache_policy
+
     async def async_get_device_manager(self) -> DeviceManager:
         """Return the DeviceManger with the current state of devices."""
         if not self._device_manager_task:
@@ -418,7 +430,7 @@ class GoogleNestSubscriber:
 
     async def _async_create_device_manager(self) -> DeviceManager:
         """Create a DeviceManager, populated with initial state."""
-        device_manager = DeviceManager()
+        device_manager = DeviceManager(self._cache_policy)
         structures = await self._api.async_get_structures()
         for structure in structures:
             device_manager.add_structure(structure)
